/**
 * An element of a {@link SelectorArray}, comprising a selector and optional
 * index. This is to support a DOM query/selector language that allows indexing.
 */
interface SelectorFragment {
    selector: string;
    index?: number;
}
/**
 * A class that represents a flexible selector, allowing intermediate indices.
 * It's an array of {@link SelectorFragment}s that represents a set of query and
 * index instructions allowing DOM queries that include indexing.
 */
declare class SelectorArray extends Array<SelectorFragment> {
    /**
     * Create a new selector array that extends this one with either another
     * selector, or an indexing operation.
     *
     * @param val the selector or index with which to extend this selector array
     */
    extend(val: string | number): SelectorArray;
    /**
     * A string representation of this selector array, e.g. `div span[1] strong`
     */
    toString(): string;
}
/**
 * A class that can execute selector-based DOM queries, with supporting for
 * intermediate indexing.
 */
export default class DOMQuery {
    readonly root: Element | null;
    readonly selectorArray: SelectorArray;
    /**
     * @param root the root element from which to query
     * @param selectorArray the selector array describing the query to execute
     */
    constructor(root: Element | null, selectorArray?: SelectorArray);
    /**
     * Query the first matching element
     */
    query(): Element | null;
    /**
     * Query all matching elements
     */
    queryAll(): Element[];
    /**
     * Create a child {@link DOMQuery} from a selector and an optional index.
     *
     * @param selector selector to query
     * @param index target index of the {@link Element} in the query results, or
     * null if this isn't an index query
     */
    createChild(selector: string, index: number | null): DOMQuery;
}
export {};
